# tests/test_decompositions.py
import numpy as np
from src.DIY_NLA.decompositions import lu_decomposition, cholesky_decomposition, qr_decomposition, svd_decomposition

def test_lu_decomposition():
    A = np.array([[4, 3], [6, 3]])
    P, L, U = lu_decomposition(A)
    assert np.allclose(P @ L @ U, A)

def test_cholesky_decomposition():
    A = np.array([[4, 12, -16], [12, 37, -43], [-16, -43, 98]])
    L = cholesky_decomposition(A)
    assert np.allclose(L @ L.T, A)

def test_qr_decomposition():
    A = np.random.rand(3, 2)
    Q, R = qr_decomposition(A)
    assert np.allclose(Q @ R, A)
    assert np.allclose(Q.T @ Q, np.eye(Q.shape[1]))  # Orthogonality

def test_svd_decomposition():
    A = np.array([[1, 2], [3, 4], [5, 6]])
    U, s, Vt = svd_decomposition(A)
    assert np.allclose(U @ np.diag(s) @ Vt, A)


# tests/test_direct_solvers.py
import numpy as np
from src.DIY_NLA.direct_solvers import solve_lu, solve_cholesky

def test_solve_lu():
    A = np.array([[3, 2], [1, 2]])
    b = np.array([5, 4])
    x = solve_lu(A, b)
    assert np.allclose(A @ x, b)

def test_solve_cholesky():
    A = np.array([[4, 1], [1, 3]])
    b = np.array([1, 2])
    x = solve_cholesky(A, b)
    assert np.allclose(A @ x, b)


# tests/test_iterative_solvers.py
import numpy as np
import pytest
from src.DIY_NLA.iterative_solvers import conjugate_gradient



def test_conjugate_gradient_larger():
    # Generate larger SPD matrix
    np.random.seed(42)
    B = np.random.rand(10, 10)
    A = B.T @ B + np.eye(10)  # Make SPD
    x_true = np.random.rand(10)
    b = A @ x_true
    
    # Solve with CG
    x = conjugate_gradient(A, b, max_iter=100, tol=1e-8)
    
    # Verify solution accuracy
    assert np.allclose(A @ x, b, atol=1e-6)
    assert np.allclose(x, x_true, atol=1e-4)




# tests/test_eigen.py
import numpy as np
from src.DIY_NLA.eigen import power_iteration

def test_power_iteration():
    A = np.array([[2, 1], [1, 2]])
    eigval, eigvec = power_iteration(A)
    # Validate eigen-equation A*v = λ*v
    assert np.allclose(A @ eigvec, eigval * eigvec, atol=1e-6)


# tests/test_basic_ops.py
import numpy as np
import pytest
from src.DIY_NLA.basic_ops import dot_product, matrix_multiply, vector_norm, matrix_norm

def test_dot_product():
    v1 = np.array([1, 2, 3])
    v2 = np.array([4, 5, 6])
    assert dot_product(v1, v2) == 32
    assert dot_product(v1, v1) == 14

def test_matrix_multiply():
    A = np.array([[1, 2], [3, 4]])
    B = np.array([[5, 6], [7, 8]])
    expected = np.array([[19, 22], [43, 50]])
    assert np.allclose(matrix_multiply(A, B), expected)

def test_vector_norm():
    v = np.array([3, 4])
    assert vector_norm(v) == 5.0
    assert vector_norm(v, p=1) == 7.0

def test_matrix_norm():
    A = np.array([[1, 2], [3, 4]])
    assert np.isclose(matrix_norm(A), np.linalg.norm(A))
    assert np.isclose(matrix_norm(A, p=np.inf), 7)


from setuptools import setup, find_packages

setup(
    name="DNLAP",
    version="0.0.3-beta",  # Will be automated later
    packages=find_packages(),
    install_requires=["numpy", "scipy"],
    python_requires=">=3.7",
)
